#include <stdio.h>
#include <unistd.h>
#include <getopt.h>
#include <string.h>

#define PAGE_SIZE 4096

const char* _version_ = "1.0";

int show_process_mem( int argc, char** argv, int optind, int siz, int shared, int show_mb, int show_kb );
void help( char* prg );

int main( int argc, char** argv ) 
{
	static struct option options[] = {
		{ 0, 0, 0, 0 }
	};
	int optidx = 0;
	int show_size = 0;
	int show_shared = 0;
	int show_mb = 0;
	int show_kb = 0;
	int h = 0;
	char c;
	while( 1 ) {
		c = getopt_long( argc, argv, "vzsmhk", options, &optidx );
		if( c == -1 ) {
			break;
		}
		switch( c ) {
			case '?':
				return 1;
			case 'z':
				show_size = 1;
				break;
			case 's':
				show_shared = 1;
				break;
			case 'm':
				show_mb = 1;
				break;
			case 'k':
				show_kb = 1;
				break;
			case 'h':
				h = 1;
				break;
			case 'v':
				printf( "%s\n", _version_ );
				return 0;
		}
	}
	if( h == 1 ) {
		help( argv[ 0 ] );
		return 0;
	} else {
		if( optind < argc ) {
			return show_process_mem( argc, argv, optind, show_size, show_shared, show_mb, show_kb );
		} else {
			fprintf( stderr, "no process id given\n" );
		}
	}
	return 1;
}

void help( char* prg )
{
	printf( "usage: %s [OPTIONS] PID\nDetermines the memory usage of a process.\nOptions:\n", prg );
	printf( "  -z  show size (default is resident)\n"
			"  -s  show shared memory\n"
			"  -m  convert to MB\n"
			"  -k  convert to KB\n"
			"  -h  this help screen\n"
			"  -v  version\n" );
}

int show_process_mem( int argc, char** argv, int optind, int siz, int shared, int show_mb, int show_kb )
{
	char buf[ 4096 ];
	int _size = 0;
	int _res = 0;
	int _shared = 0;
	int ret = 0;
	int pid;
	for( ; optind < argc; ++optind ) {
		pid = atoi( argv[ optind ] );
		sprintf( buf, "/proc/%d/statm", pid );
		FILE* f = fopen( buf, "r" );
		if( f ) {
			fscanf( f, "%d %d %d", &_size, &_res, &_shared );
			fclose( f );
			if( siz ) {
				ret = _size;
			} else if( shared ) {
				ret = _shared;
			} else {
				ret = _res;
			}
			ret *= PAGE_SIZE;
			if( show_mb ) {
				ret /= ( 1024 * 1024 );
			} else if( show_kb ) {
				ret /= 1024;
			}
			printf( "%d ", ret );
		} else {
			printf( "- " );
			ret = 1;
		}
	}
	printf( "\n" );
	return ret;
}
